#!/usr/bin/ruby -w
#
# allmusic - www.allmusic.com implementation of MusicSite
#
# version: $Id: allmusic.rb 215 2004-02-07 18:35:41Z kapheine $
#
# Copyright (C) 2003 Zachary P. Landau <kapheine@hypa.net>
# All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

require 'musicextras/musicsite'
require 'musicextras/album'
require 'musicextras/artist'
require 'cgi'

module MusicExtras

  ### This is an implementation of MusicSite for www.allmusic.com
  class AllMusic < MusicSite

    attr_accessor :artist, :album # :nodoc:

    def initialize
      super('AllMusic', 'www.allmusic.com')

      Artist::register_plugin(self, :biography, '{ARTIST}/biography.txt')
      Artist::register_plugin(self, :image, '{ARTIST}/{ARTIST}.img')
      Artist::register_plugin(self, :years_active, '{ARTIST}/years_active.txt')
      Album::register_plugin(self, :cover, '{ARTIST}/covers/{ALBUM}.img')
    end

    # Fetches artist image from site, returning the image as a binary string
    # [+artist+] an Artist object
    #
    # Note: returns nil if somethign went wrong (including not being able to
    # find the artist image)
    def image(new_artist)
      @artist = new_artist

      unless @artist.name
	info("image: artist not specified")
	return nil
      end

      page = get_artist_page
      return nil unless page
      page.scan(/<span id="artpic"><img src=http:\/\/image.allmusic.com(.*?) width=200 hspace=0 vspace=0 align=left name="artistpic"><\/span>/) do |url|
	return fetch_page("http://image.allmusic.com/#{url}")
      end 

      nil
    end

    # Fetches album cover from the site, returning the image as a binary string
    # [+album+] an Album Object
    #
    # Note: returns nil if something went wrong
    def cover(new_album)
      @album = new_album
      @artist = new_album.artist

      unless @album.title and @album.artist.name
	info("cover: album title or artist not specified")
	return nil
      end

      url = get_album_cover_url
      return nil unless url
      return fetch_page("http://image.allmusic.com/#{url}")
    end

    # Fetches artist biography info, returning nil if none is found
    # [+artist+] an Artist object
    # Calls get_artist_page
    def biography(new_artist)
      @artist = new_artist

      page = get_artist_page()
      return nil if !page

      # we add a few tabs to make the output a little nicer.
      text = extract_text(page.gsub('<P>', "<P>\t"), /<div.*align=left>(.*)<\/div>/) 
      unless text then text = extract_text(page.gsub('<P>', "<P>\t"), /<div align=left>(.*?)<\/td>/) end
      if text
	return "\t" + text
      else
	return nil
      end
    end

    # Fetches the years the artist was active for, returning nil if not found
    # [+artist+] an Artist object
    def years_active(new_artist)
      @artist = new_artist

      page = get_artist_page()
      return nil if !page

      active = nil

      page.scan(/javascript:z\('L(\d)/) do |d|
	active = '' unless active
	active += "#{d}0s "
      end

      return active
    end

    # Returns the page allmusic returns in a search for the artist.
    # Returns nil if something went wrong (including just not finding the page)
    def get_artist_page # :nodoc:
      post = "sql=#{CGI::escape(@artist.name)}&P=amg&opt1=1"

      body = fetch_page("/cg/amg.dll?#{post}")

      # this means the artist wasn't found
      if body =~ /.*artists with names like.*/
	info("get_artist_page: artist page for #{@artist.name} not found")
	return nil
      else
	return body
      end

    end

    # Fetches the url for an album or returns nil if its not found
    # Calls get_artist_page
    def get_album_url # :nodoc:
      page = get_artist_page()
      return nil if !page
      page.scan(/(?:(?:<\/font>)|(?:&nbsp;))<a class=a href="javascript:z\('(.*?)'\)"[^>]*>(.*?)<\/a>/mi) do |url, name|
	return "/cg/amg.dll?p=amg&sql=#{url}" if match?(@album.title, name, /^The /i)
      end

      info("get_album_url: album url for #{@album.title} by #{@album.artist.name} not found")
      return nil
    end

    # Fetches the album cover url. Calls get_album_url
    def get_album_cover_url # :nodoc:
      album_url = get_album_url()
      return nil if !album_url
      page = fetch_page(album_url)
      return nil if !page
      page.scan(/<IMG border=0 width=200 align=left src=http:\/\/image.allmusic.com(.*?) hspace=10>/) do |url|
	return url[0]
      end
      page.scan(/<IMG width=200 ALIGN=TOP src=http:\/\/image.allmusic.com(.*?)>/) do |url|
	return url[0]
      end

      info("get_album_cover_url: could not find cover for #{@album.title} by #{@album.artist.name}")
      return nil
    end

  end
end

MusicExtras::AllMusic.new

