#!/usr/bin/ruby -w
# :nodoc all
#
# Test for Data Accessor
#

require 'test/unit'
require 'musicextras/dataaccessor'

include MusicExtras

class TestPlugin
  def initialize
    TestSong::register_plugin(self, :get_lyrics, '/TestPlugin/cache')
    TestSong::register_plugin(self, :get_lyrics2, '/TestPlugin/cache2')
  end

  def get_lyrics(song)
    return nil
  end

  def get_lyrics2(song)
    return 'blah'
  end
end

class TestPlugin2
  def initialize
    TestSong::register_plugin(self, :get_lyrics, '/TestPlugin2/cache')
  end

  def get_lyrics(song)
    return "TestPlugin2: #{song.title}"
  end
end

class TestPlugin3
  def initialize
    TestArtist::register_plugin(self, :get_artist_image, '/TestPlugin3/cache')
  end

  def get_artist_image(artist)
    return "TestPlugin3: #{artist.name}"
  end
end

class TestSong < DataAccessor
  attr_accessor :title
  def initialize(title)
    super(false)
    @title = title
  end
end

class TestArtist < DataAccessor
  attr_accessor :name
  def initialize(name)
    super(false)
    @name = name
  end
end

class TC_DataAccessor < Test::Unit::TestCase

  def setup
    @song = TestSong.new('MyTitle')
    @artist = TestArtist.new('MyArtist')
  end

  def test_plugins
    TestPlugin.new
    assert_equal(nil, @song.get_lyrics)
    TestPlugin2.new
    assert_equal('TestPlugin2: MyTitle', @song.get_lyrics)
    TestPlugin3.new
    assert_equal('TestPlugin3: MyArtist', @artist.get_artist_image)
    assert_equal('TestPlugin2: MyTitle', @song.get_lyrics)

    assert_equal([:get_lyrics, :get_lyrics2], TestSong::accessors)
    assert_equal([:get_artist_image], TestArtist::accessors)

    assert_raises(DataAccessor::AccessorNotImplemented) { @song.get_artist_image }
    assert_raises(DataAccessor::AccessorNotImplemented) { @artist.get_lyrics }
  end

  def test_cache_values
    da = DataAccessor.new(false)
    assert_equal('blah', da.cache_values('blah'))
  end

end
