#!/usr/bin/ruby -w
# :nodoc all
#
# Test for MusicSite
#

require 'test/unit'
require 'musicextras/musicsite'

include MusicExtras

class TC_MusicSite < Test::Unit::TestCase

  def setup
    @site = MusicSite.new('MusicSite', 'MusicSiteDefaultURL')
  end

  def test_attributes
    assert_equal('MusicSite', @site.name)
    assert_equal('MusicSiteDefaultURL', @site.url)

    assert_equal('Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.2.1) Gecko/20030225', MusicSite::USERAGENTS['Mozilla'])
    assert_equal('Opera/6.0 (Linux 2.4.18 i686; U)  [en]', MusicSite::USERAGENTS['Opera'])
    assert_equal('Lynx/2.8.4dev.16 libwww-FM/2.14 SSL-MM/1.4.1 OpenSSL/0.9.6-beta3', MusicSite::USERAGENTS['Lynx'])
  end

  def test_to_s
    assert_equal('MusicSite: MusicSiteDefaultURL', @site.to_s)
  end

  def test_match
    assert(@site.match?('nospaces', 'no spaces'))
  end

  
  def test_extract_text
    html = "HEADER JUNK\nMORE HEADERS\nThe Meat and Potatoes\n" +
           "FOOTER JUNK\nMORE FOOTER JUNK"
    assert_equal('The Meat and Potatoes', @site.extract_text(html,
                      /MORE HEADERS\n(.*?)\nFOOTER JUNK/m))

    html = "<html>\n<body>\n<p class=\"text\"><br>" +
           "all work and no play make<br>" +
           " homer something something<br>" +
           "<table>\n<tr>\n<td><br>" +
           "There was a young man from Spleen<br>" +
           "Who invented a wanking machine<br>" +
	   "On the 99th stroke, the fucking thing broke<br>" +
	   "And whipped his balls into cream<br>" +
	   "</td></tr></table>" +
	   "go crazy? don't mind if i do!<br>\n" +
	   "</body></html>\n"

    assert_equal("There was a young man from Spleen\n" +
                 "Who invented a wanking machine\n" +
		 "On the 99th stroke, the fucking thing broke\n" +
		 "And whipped his balls into cream\n",
		 @site.extract_text(html, /<table>\n(.*?)<\/td>/m))

    html = "whatever\n\n\n\nblahblah\n\n\n\n"
    assert_equal("whatever\n\nblahblah\n", 
                  @site.extract_text(html, /(.*)\n/m))

    html = "one\n\ntwo\n\nthree\n\nfour\n\n"
    assert_equal("one\ntwo\nthree\nfour\n",
          	  @site.extract_text(html, /(.*)\n/m))
  end

  class TestSite < MusicSite
    def initialize
      super('TestSite', 'www.google.com')
    end
  end

  class InvalidTestSite < MusicSite
    def initialize
      super('InvalidTestSite', 'www.idontexistandhoepfullyneverwill.info')
    end
  end

  def test_fetch_page
    @t = TestSite.new
    assert_match(/.*Try our latest demos.*/, @t.fetch_page('/about.html'))
    assert_match(/.*Global Preferences.*/, @t.fetch_page('/preferences?hl=en'))

    @i = InvalidTestSite.new
    assert_nil(@i.fetch_page('/whatever.html'))

    # we would check for invalid .coms, but oh yeah, fuck verisign
  end

end
