#!/usr/bin/ruby -w
# :nodoc all
#
# Test For AllMusic

require 'test/unit'
require 'musicextras/musicsites/allmusic'
require 'digest/md5'

include MusicExtras

class TC_AllMusic < Test::Unit::TestCase

  def setup
    @site = AllMusic.new
    @md5 = Digest::MD5

    @site.artist = Artist.new('')
    @site.album = Album.new('', '')
  end

  def test_inheritence
    assert_kind_of(MusicSite, @site)
  end

  def test_class_vars
    assert_equal('AllMusic', @site.name)
    assert_equal('www.allmusic.com', @site.url)
  end

  def test_get_artist_page
    @site.artist.name = 'Refused'
    assert_match(/.*Swedish hardcore band Refused.*/, @site.get_artist_page)

    @site.artist.name = 'ASDFBLAH'
    assert_nil(@site.get_artist_page)

    @site.artist.name = 'they might be giants'
    assert_match(/.*Combining a knack.*/, @site.get_artist_page)

    @site.artist.name = '59 Times The Pain'
    assert_match(/.*Named in honor.*/, @site.get_artist_page)

    @site.artist.name = 'Forgotten'
    assert_match(/.*Creating radical themes.*/, @site.get_artist_page)
  end

  def test_get_artist_image
    @site.artist.name = 'Refused'
    image = @site.image(@site.artist)
    assert_equal('cedfbfbbf7f57cee2abf7953ffe6e2c0',
    @md5.digest(image).unpack("H*").to_s)

    @site.artist.name = 'They Might Be Giants'
    image = @site.image(@site.artist)
    assert_equal('160b112097c6e33ef67702769f13741f',
    @md5.digest(image).unpack("H*").to_s)
  end

  def test_get_album_url
    # url changes every time, just make sure the url isn't nil
    @site.artist.name = 'Refused'
    @site.album.title = 'Songs To Fan The Flames Of Discontent'
    assert(@site.get_album_url)

    @site.artist.name = 'Pistol Grip'
    @site.album.title = 'The Shots From The Kalico Rose'
    assert(@site.get_album_url)

    @site.artist.name = 'Pistol Grip'
    @site.album.title = 'Shots From The Kalico Rose'
    assert(@site.get_album_url)

    @site.artist.name = 'Total Chaos'
    @site.album.title = 'Anthems from the Alleyway'
    assert(@site.get_album_url)
  end

  def test_get_album_cover_url
    @site.artist.name = 'Refused'
    @site.album.title = 'Songs To Fan The Flames Of Discontent'
    assert_equal('/00/amg/cov200/drc900/c945/c94507b332s.jpg', @site.get_album_cover_url)

    @site.artist.name = 'Pistol Grip'
    @site.album.title = 'Shots From The Kalico Rose'
    assert_equal('/00/amg/cov200/dre700/e757/e75709ddswo.jpg', @site.get_album_cover_url)
  end

  def test_get_album_cover
    @site.artist = nil
    @site.album.title = 'Songs To Fan The Flames Of Discontent'
    @site.album.artist.name = 'Refused'
    cover = @site.cover(@site.album)
    assert_equal('a3002a499c5444d7fb70720467f0a5f8',
    @md5.digest(cover).unpack("H*").to_s)

    @site.album.title = 'Shots From The Kalico Rose'
    @site.album.artist.name = 'Pistol Grip'
    cover = @site.cover(@site.album)
    assert_equal('5903f80a6cec2e74bec70f1cee1840f8',
    @md5.digest(cover).unpack("H*").to_s)
  end

  def test_get_artist_bio
    @site.artist.name = 'Bad Religion'
    assert_match(/.*Out of all.*of Belief.*/m, @site.biography(@site.artist))
    # XXX: no workie
    #@site.artist.name = '59 Times The Pain'
    #assert_match(/.*Named in honor.*/, @site.biography(@site.artist))

    @site.artist.name = 'They Might Be Giants'
    assert_match(/.*Combining.*, No!.*/m, @site.biography(@site.artist))

    @site.artist.name = 'Refused'
    assert_match(/.*The Swedish hardcore band.*in music.*/m, @site.biography(@site.artist))

    @site.artist.name = 'Forgotten'
    assert_match(/.*Creating radical.*in May 2003.*/m, @site.biography(@site.artist))

    @site.artist.name = 'Doesnt exist'
    assert_nil(@site.biography(@site.artist))
  end

end
